﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Test.zuora;

namespace Test
{

    class Program
    {

        static string USERNAME = "username";
        static string PASSWORD = "password";
        static string ENDPOINT = "https://www.zuora.com/apps/services/a/17.0";
        static string ACCOUNTING_CODE = "TestNewComponent";

        static bool testSubscribeFlag = true;

        private zuora.ZuoraService binding;

        static void Main(string[] args)
        {
            Program p = new Program();
            p.login();

            if (testSubscribeFlag)
            {
                p.testSubscribe();
            }
            else
            {
                p.testAccountCRUD();
            }

        }

        public Program()
        {
            //create a new instance of the web service proxy class 
            binding = new zuora.ZuoraService();
            binding.Url = ENDPOINT;
        }

        private bool login()
        {

            try
            {
                //execute the login placing the results  
                //in a LoginResult object 
                zuora.LoginResult loginResult = binding.login(USERNAME, PASSWORD);

                //set the session id header for subsequent calls 
                binding.SessionHeaderValue = new zuora.SessionHeader();
                binding.SessionHeaderValue.session = loginResult.Session;

                //reset the endpoint url to that returned from login 
                // binding.Url = loginResult.ServerUrl;

                print("Session: " + loginResult.Session);
                print("ServerUrl: " + loginResult.ServerUrl);

                return true;
            }
            catch (Exception ex)
            {
                //Login failed, report message then return false 
                Console.WriteLine("Login failed with message: " + ex.Message);
                return false;
            }
        }

        private void testAccountCRUD()
        {
            string accId = createAccount(true);

            Account accountQuery = queryAccount(accId);
            print("Create Active Account: " + accId);
            // print(toString(accountQuery));

            Account accUpdate = new Account();
            accUpdate.Id = accId;
            accUpdate.Name = "testUpdate" + DateTime.Now.Ticks;

            update(accUpdate);

            accountQuery = queryAccount(accId);
            print("Updated Account: " + accId);
            // print(toString(accountQuery));

            bool deleted = delete("Account", accId);
            print("Deleted Account: " + deleted);

        }

        public string createAccount(bool active)
        {

            // create account
            Account acc1 = makeAccount();
            string accountId = create(acc1);

            if (active)
            {

                // create contact
                Contact con = makeContact();
                con.AccountId = accountId;
                string contactId = create(con);

                PaymentMethod pm = makePaymentMethod();
                pm.AccountId = accountId;
                string pmId = create(pm);

                // set required active fields and activate
                Account accUpdate = new Account();
                accUpdate.Id = accountId;
                accUpdate.Status = "Active";
                accUpdate.SoldToId = contactId;
                accUpdate.BillToId = contactId;
                accUpdate.AutoPay = true;
                accUpdate.PaymentTerm = "Due Upon Receipt";
                accUpdate.DefaultPaymentMethodId = pmId;
                update(accUpdate);
            }

            return accountId;
        }

        private string create(zObject acc)
        {
            SaveResult[] result = binding.create(new zObject[] { acc });
            return result[0].Id;
        }

        private Account queryAccount(string accId)
        {
            QueryResult qResult = binding.query("SELECT id, name, accountnumber FROM account WHERE id = '" + accId + "'");
            Account rec = (Account)qResult.records[0];
            return rec;
        }

        private string update(zObject acc)
        {
            SaveResult[] result = binding.update(new zObject[] { acc });
            return result[0].Id;
        }

        private bool delete(String type, string id)
        {

            DeleteResult[] result = binding.delete(type, new string[] { id });
            return result[0].success;

        }

        private void testSubscribe()
        {

            ProductRatePlanCharge charge = getChargeByAccountingCode(ACCOUNTING_CODE);

            Account acc = makeAccount();
            Contact con = makeContact();
            PaymentMethod pm = makePaymentMethod();
            Subscription subscription = makeSubscription();

            SubscriptionData sd = new SubscriptionData();
            sd.Subscription = subscription;

            RatePlanData[] subscriptionRatePlanDataArray = makeRatePlanData(charge);
            sd.RatePlanData = subscriptionRatePlanDataArray;

            SubscribeRequest sub = new SubscribeRequest();
            sub.Account = acc;
            sub.BillToContact = con;
            sub.PaymentMethod = pm;
            sub.SubscriptionData = sd;

            SubscribeRequest[] subscribes = new SubscribeRequest[1];
            subscribes[0] = sub;

            SubscribeResult[] result = binding.subscribe(subscribes);
            print(createMessage(result));

        }

        /**
          * Creates the print format of the subscribe result value.
          * @param resultArray
          * @return
          */
        private String createMessage(SubscribeResult[] resultArray)
        {
            string resultString = "SusbscribeResult :\n";
            if (resultArray != null)
            {
                SubscribeResult result = resultArray[0];
                if (result.Success)
                {
                    resultString = resultString + "\nSubscribe Result: \n";
                    resultString = resultString + "\n\tAccount Id: " + result.AccountId;
                    resultString = resultString + "\n\tAccount Number: " + result.AccountNumber;
                    resultString = resultString + "\n\tSubscription Id: " + result.SubscriptionId;
                    resultString = resultString + "\n\tSubscription Number: " + result.SubscriptionNumber;
                    resultString = resultString + "\n\tInvoice Number: " + result.InvoiceNumber;
                    resultString = resultString + "\n\tPayment Transaction: " + result.PaymentTransactionNumber;
                }
                else
                {
                    resultString = resultString + "\nSubscribe Failure Result: \n";
                    Error[] errors = result.Errors;
                    if (errors != null)
                    {
                        foreach (Error error in errors)
                        {
                            resultString = resultString + "\n\tError Code: " + error.Code;
                            resultString = resultString + "\n\tError Message: " + error.Message;
                        }
                    }
                }
            }
            return resultString;
        }


        /**
         * Creates a Subscription object reading the values from the property
         * @return
         */
        private Subscription makeSubscription()
        {

            DateTime calendar = DateTime.Now;

            Subscription sub = new Subscription();
            sub.Name = "SomeSubscription" + DateTime.Now.Ticks;
            sub.TermStartDate = calendar;
            sub.ContractEffectiveDate = calendar;
            sub.ContractAcceptanceDate = calendar;
            sub.ServiceActivationDate = calendar;

            sub.InitialTerm = 12;
            sub.InitialTermSpecified = true;
            sub.RenewalTerm = 12;
            sub.RenewalTermSpecified = true;
            sub.Notes = "This is a test subscription";

            return sub;
        }

        private PaymentMethod makePaymentMethod()
        {

            DateTime theDate = DateTime.Now;
            theDate.AddYears(1);

            PaymentMethod pm = new PaymentMethod();
            pm.Type = "CreditCard";
            pm.CreditCardType = "Visa";
            pm.CreditCardAddress1 = "52 Vexford Lane";
            pm.CreditCardCity = "Anaheim";
            pm.CreditCardState = "California";
            pm.CreditCardPostalCode = "92808";
            pm.CreditCardCountry = "United States";
            pm.CreditCardHolderName = "Firstly Lastly";
            pm.CreditCardExpirationYear = theDate.Year;
            pm.CreditCardExpirationYearSpecified = true;
            pm.CreditCardExpirationMonth = 12;
            pm.CreditCardExpirationMonthSpecified = true;
            pm.CreditCardNumber = "4111111111111111";

            return pm;
        }

        public static Contact makeContact()
        {
            long time = DateTime.Now.Ticks;
            Contact con = new Contact();
            con.FirstName = "Firstly" + time;
            con.LastName = "Secondly" + time;
            con.Address1 = "52 Vexford Lane";
            con.City = "Anaheim";
            con.State = "California";
            con.Country = "United States";
            con.PostalCode = "92808";
            con.WorkEmail = "contact@test.com";
            con.WorkPhone = "4152225151";
            return con;
        }

        private Account makeAccount()
        {
            long time = DateTime.Now.Ticks;
            Account acc = new Account();
            acc.AccountNumber = "t-" + time; // string
            acc.Batch = "Batch1"; // enum
            acc.BillCycleDay = 1; // int
            acc.BillCycleDaySpecified = true;
            acc.AllowInvoiceEdit = true; // boolean
            acc.AutoPay = false;
            acc.CrmId = "SFDC-" + time;
            acc.Currency = "USD"; // standard DB enum
            acc.CustomerServiceRepName = "CSR Dude";
            acc.Name = "SomeAccount" + time;
            acc.PurchaseOrderNumber = "PO-" + time;
            acc.SalesRepName = "Sales Dude";
            acc.PaymentTerm = "Due Upon Receipt";
            acc.Status = "Draft";
            return acc;
        }

        private RatePlanData[] makeRatePlanData(ProductRatePlanCharge charge)
        {

            RatePlanData ratePlanData = new RatePlanData();

            RatePlan ratePlan = new RatePlan();
            ratePlanData.RatePlan = ratePlan;
            ratePlan.AmendmentType = "NewProduct";
            ratePlan.ProductRatePlanId = charge.ProductRatePlanId;

            RatePlanChargeData ratePlanChargeData = new RatePlanChargeData();
            ratePlanData.RatePlanChargeData = new RatePlanChargeData[] { ratePlanChargeData };

            RatePlanCharge ratePlanCharge = new RatePlanCharge();
            ratePlanChargeData.RatePlanCharge = ratePlanCharge;

            ratePlanCharge.ProductRatePlanChargeId = charge.Id;
            ratePlanCharge.Quantity = 1;
            ratePlanCharge.QuantitySpecified = true;
            ratePlanCharge.TriggerEvent = "ServiceActivation";

            return new RatePlanData[] { ratePlanData };
        }

        private ProductRatePlanCharge getChargeByAccountingCode(String accountingCode)
        {

            QueryResult result = binding.query("select Id, ProductRatePlanId from ProductRatePlanCharge where AccountingCode = '" + accountingCode + "'");
            ProductRatePlanCharge rec = (ProductRatePlanCharge)result.records[0];
            return rec;

        }

        private void print(string p)
        {
            Console.WriteLine(p);
        }

    }
}
